class UnsafeBehaviorError(Exception):
    pass


class GlobalQueryException(Exception):
    pass


class SchemaError(ValueError):
    pass


class ValidationError(Exception):
    pass


class ModifyFailedError(Exception):
    pass
